module SurveyMonkey
  class Request
    attr_accessor :api_method, :uri, :request, :parameters, :response

    def initialize(options={})
      @auth             = options[:auth] || Auth.new
      @api_method       = options[:api_method].to_s
      @init_parameters  = options[:params]

      @api_key = @auth.api_key
      @access_token = @auth.access_token

      ### Instantiate Parameters based on API method
      @parameters = Parameters.new(@api_method)
      @uri = @parameters.uri

      # If options[:params] was spet, set parameters now
      if @init_parameters.respond_to? :each
        set_parameters do |p|
          @init_parameters.each do |key,value|
            p.send "#{key}=", value
          end
        end
      end
    end

    def set_parameters
      yield @parameters if block_given?
    end

    def request_body
      ### Params to be included in Body of request, should be a JSON encoded string
      @parameters.to_json
    end

    def request_method
      @parameters.http_method
    end

    def request_params
      ### Params to be URL Encoded
      Hash['api_key' => api_key]
    end

    def request_header
      ### Header includes content-type and access token
      Hash["Authorization" => "bearer #{access_token}", "Content-Type" => "application/json"]
    end

    def request_url
      uri
    end

    def build_request
      @request = Typhoeus::Request.new(request_url, method: request_method, body: request_body, params: request_params, headers: request_header)
    end

    def run_request
      request = build_request
      request.on_complete do |response|
        if response.success?
          @response = response
        elsif response.timed_out?
          @response = "Response timed out"
        elsif response.code == 0
          # Could not get an http response, something's wrong.
          @response = response.return_message
        else
          # Received a non-successful http response.
          @response = "HTTP request failed: #{response.code}"
        end
      end
      request.run
      self
    end

    def set_on_complete_response
      @request.on_complete do |response|
        if response.success?
          @response = response
        elsif response.timed_out?
          @response = "Response timed out"
        elsif response.code == 0
          # Could not get an http response, something's wrong.
          @response = response.return_message
        else
          # Received a non-successful http response.
          @response = "HTTP request failed: #{response.code}"
        end
      end
    end

    def result
      JSON.parse @response.body
    end

    def submit
      run_request
      result
    end

    private
      def api_key
        @api_key
      end

      def access_token
        @access_token
      end
  end
end